// ----------------------------------------------------------------------------
/**
 * \file	main.c
 * \brief	SJA1000 Testprogram
 *
 * \author	Fabian Greif <fabian.greif@rwth-aachen.de>
 * \version $Id: main.c 7255 2009-02-01 13:11:30Z fabian $
 */
// ----------------------------------------------------------------------------

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

#include <util/delay.h>
#include <string.h>

#include "can.h"
#include "utils.h"

// ----------------------------------------------------------------------------
// main-loop

int main(void)
{
	// Release PC2-PC7
	SFIOR = (1<<XMM2)|(1<<XMM1);
	
	// Enable external-memory, one wait-state
	MCUCR |= (1<<SRE)|(SRW10);
	EMCUCR = (SRW00);
	
	// Init the i/o-pins
	DDRB = 0x17;
	PORTB = 0xff;		// LEDs on
	
	DDRC = 	0x03;
	PORTC = 0xfc;
	
	DDRD = 0xc2;
	PORTD = 0xfc;
	
	DDRA = 0;
	PORTA = 0;
	
	DDRE = 0x06;
	PORTE = 0x05;
	
	// init sja1000 interface
	if (!can_init(BITRATE_125_KBPS))
	{
		// If the initialization fails the three red leds will stay on ...
		for(;;);
	}
	
	// ... otherwise there is only a short pulse visible
	_delay_ms(20);
	
	// Switch off red LEDs
	PORTB &= ~((1<<PB0)|(1<<PB1)|(1<<PB4));
	
	// Create a test messsage
	can_t msg;
	
	msg.id = 0x123456;
	msg.flags.rtr = 0;
	msg.flags.extended = 1;
	
	msg.length = 4;
	msg.data[0] = 0xde;
	msg.data[1] = 0xad;
	msg.data[2] = 0xbe;
	msg.data[3] = 0xef;
	
	// Send the message
	can_send_message(&msg);
	
	while (1)
	{
		// Check if a new messag was received
		if (can_check_message())
		{
			can_t msg;
			
			// Try to read the message
			if (can_get_message(&msg))
			{
				// If we received a message resend it with a different identifier
				msg.id += 10;
				
				if (msg.flags.extended) {
					TOGGLE(LED1);
				}
				else {
					TOGGLE(LED2);
				}
				
				// Send the new message
				can_send_message(&msg);
			}
		}
	}
}
